@@ -34,7 +34,6 @@ import (
 	fakerouteinformer "knative.dev/serving/pkg/client/injection/informers/serving/v1/route/fake"
 
 	"github.com/google/go-cmp/cmp"
-	"github.com/google/uuid"
 	"golang.org/x/sync/errgroup"
 
 	corev1 "k8s.io/api/core/v1"
@@ -46,12 +45,15 @@ import (
 	"k8s.io/client-go/tools/record"
 
 	network "knative.dev/networking/pkg"
+	"knative.dev/networking/pkg/apis/networking"
 	"knative.dev/networking/pkg/apis/networking/v1alpha1"
 	"knative.dev/pkg/configmap"
 	"knative.dev/pkg/controller"
+	pkgnet "knative.dev/pkg/network"
 	"knative.dev/pkg/ptr"
 	"knative.dev/pkg/reconciler"
 	"knative.dev/pkg/system"
+	cfgmap "knative.dev/serving/pkg/apis/config"
 	"knative.dev/serving/pkg/apis/serving"
 	v1 "knative.dev/serving/pkg/apis/serving/v1"
 	"knative.dev/serving/pkg/gc"
@@ -115,6 +117,11 @@ func newTestSetup(t *testing.T, opts ...reconcilerOption) (
 			Name:      gc.ConfigName,
 			Namespace: system.Namespace(),
 		},
+	}, {
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      cfgmap.FeaturesConfigName,
+			Namespace: system.Namespace(),
+		},
 	}} {
 		configMapWatcher.OnChange(cfg)
 	}
@@ -135,7 +142,7 @@ func getRouteIngressFromClient(ctx context.Context, t *testing.T, route *v1.Rout
 			serving.RouteNamespaceLabelKey: route.Namespace,
 		}).String(),
 	}
-	ingresses, err := fakenetworkingclient.Get(ctx).NetworkingV1alpha1().Ingresses(route.Namespace).List(opts)
+	ingresses, err := fakenetworkingclient.Get(ctx).NetworkingV1alpha1().Ingresses(route.Namespace).List(ctx, opts)
 	if err != nil {
 		t.Errorf("Ingress.Get(%v) = %v", opts, err)
 	}
@@ -152,7 +159,7 @@ func addRouteToInformers(ctx context.Context, t *testing.T, route *v1.Route) {
 
 	ns := route.Namespace
 
-	route, err := fakeservingclient.Get(ctx).ServingV1().Routes(ns).Get(route.Name, metav1.GetOptions{})
+	route, err := fakeservingclient.Get(ctx).ServingV1().Routes(ns).Get(ctx, route.Name, metav1.GetOptions{})
 	if err != nil {
 		t.Errorf("Route.Get(%v) = %v", route.Name, err)
 	}
@@ -176,7 +183,7 @@ func TestCreateRouteForOneReserveRevision(t *testing.T) {
 	rev := Revision(testNamespace, "test-rev", MarkRevisionReady,
 		MarkInactive("NoTraffic", "no message"))
 
-	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(rev)
+	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(ctx, rev, metav1.CreateOptions{})
 	fakerevisioninformer.Get(ctx).Informer().GetIndexer().Add(rev)
 
 	// A route targeting the revision
@@ -185,7 +192,7 @@ func TestCreateRouteForOneReserveRevision(t *testing.T) {
 		ConfigurationName: "test-config",
 		Percent:           ptr.Int64(100),
 	}), WithRouteLabel(map[string]string{"route": "test-route"}))
-	fakeservingclient.Get(ctx).ServingV1().Routes(testNamespace).Create(route)
+	fakeservingclient.Get(ctx).ServingV1().Routes(testNamespace).Create(ctx, route, metav1.CreateOptions{})
 	// Since Reconcile looks in the lister, we need to add it to the informer
 	fakerouteinformer.Get(ctx).Informer().GetIndexer().Add(route)
 
@@ -208,7 +215,9 @@ func TestCreateRouteForOneReserveRevision(t *testing.T) {
 		TLS: []v1alpha1.IngressTLS{},
 		Rules: []v1alpha1.IngressRule{{
 			Hosts: []string{
-				"test-route.test.svc.cluster.local",
+				"test-route.test",
+				"test-route.test.svc",
+				pkgnet.GetServiceHostname("test-route", "test"),
 			},
 			Visibility: v1alpha1.IngressVisibilityClusterLocal,
 			HTTP: &v1alpha1.HTTPIngressRuleValue{
@@ -225,7 +234,6 @@ func TestCreateRouteForOneReserveRevision(t *testing.T) {
 							"Knative-Serving-Namespace": testNamespace,
 						},
 					}},
-					Timeout: &metav1.Duration{Duration: 48 * time.Hour},
 				}},
 			},
 		}, {
@@ -247,18 +255,17 @@ func TestCreateRouteForOneReserveRevision(t *testing.T) {
 							"Knative-Serving-Namespace": testNamespace,
 						},
 					}},
-					Timeout: &metav1.Duration{Duration: 48 * time.Hour},
 				}},
 			},
 		}},
 	}
 	if diff := cmp.Diff(expectedSpec, ci.Spec); diff != "" {
-		t.Errorf("Unexpected rule spec diff (-want +got): %s", diff)
+		t.Error("Unexpected rule spec diff (-want +got):", diff)
 	}
 
 	// Update ingress loadbalancer to trigger placeholder service creation.
 	ci.Status = v1alpha1.IngressStatus{
-		LoadBalancer: &v1alpha1.LoadBalancerStatus{
+		DeprecatedLoadBalancer: &v1alpha1.LoadBalancerStatus{
 			Ingress: []v1alpha1.LoadBalancerIngressStatus{{
 				DomainInternal: "test-domain",
 			}},
@@ -301,7 +308,7 @@ func TestCreateRouteWithMultipleTargets(t *testing.T) {
 	}()
 	// A standalone revision.
 	rev := Revision(testNamespace, "test-rev", MarkRevisionReady)
-	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(rev)
+	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(ctx, rev, metav1.CreateOptions{})
 	fakerevisioninformer.Get(ctx).Informer().GetIndexer().Add(rev)
 
 	// A configuration and associated revision. Normally the revision would be
@@ -310,10 +317,10 @@ func TestCreateRouteWithMultipleTargets(t *testing.T) {
 	cfgrev := revisionForConfig(config)
 	config.Status.SetLatestCreatedRevisionName(cfgrev.Name)
 	config.Status.SetLatestReadyRevisionName(cfgrev.Name)
-	fakeservingclient.Get(ctx).ServingV1().Configurations(testNamespace).Create(config)
+	fakeservingclient.Get(ctx).ServingV1().Configurations(testNamespace).Create(ctx, config, metav1.CreateOptions{})
 	// Since Reconcile looks in the lister, we need to add it to the informer
 	fakecfginformer.Get(ctx).Informer().GetIndexer().Add(config)
-	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(cfgrev)
+	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(ctx, cfgrev, metav1.CreateOptions{})
 	fakerevisioninformer.Get(ctx).Informer().GetIndexer().Add(cfgrev)
 
 	// A route targeting both the config and standalone revision.
@@ -325,7 +332,7 @@ func TestCreateRouteWithMultipleTargets(t *testing.T) {
 			RevisionName: rev.Name,
 			Percent:      ptr.Int64(10),
 		}))
-	fakeservingclient.Get(ctx).ServingV1().Routes(testNamespace).Create(route)
+	fakeservingclient.Get(ctx).ServingV1().Routes(testNamespace).Create(ctx, route, metav1.CreateOptions{})
 	// Since Reconcile looks in the lister, we need to add it to the informer.
 	fakerouteinformer.Get(ctx).Informer().GetIndexer().Add(route)
 
@@ -337,7 +344,9 @@ func TestCreateRouteWithMultipleTargets(t *testing.T) {
 		TLS: []v1alpha1.IngressTLS{},
 		Rules: []v1alpha1.IngressRule{{
 			Hosts: []string{
-				"test-route.test.svc.cluster.local",
+				"test-route.test",
+				"test-route.test.svc",
+				pkgnet.GetServiceHostname("test-route", "test"),
 			},
 			HTTP: &v1alpha1.HTTPIngressRuleValue{
 				Paths: []v1alpha1.HTTPIngressPath{{
@@ -364,7 +373,6 @@ func TestCreateRouteWithMultipleTargets(t *testing.T) {
 							"Knative-Serving-Namespace": testNamespace,
 						},
 					}},
-					Timeout: &metav1.Duration{Duration: 48 * time.Hour},
 				}},
 			},
 			Visibility: v1alpha1.IngressVisibilityClusterLocal,
@@ -397,7 +405,6 @@ func TestCreateRouteWithMultipleTargets(t *testing.T) {
 							"Knative-Serving-Namespace": testNamespace,
 						},
 					}},
-					Timeout: &metav1.Duration{Duration: 48 * time.Hour},
 				}},
 			},
 			Visibility: v1alpha1.IngressVisibilityExternalIP,
@@ -405,7 +412,7 @@ func TestCreateRouteWithMultipleTargets(t *testing.T) {
 	}
 
 	if diff := cmp.Diff(expectedSpec, ci.Spec); diff != "" {
-		t.Errorf("Unexpected rule spec diff (-want +got): %v", diff)
+		t.Error("Unexpected rule spec diff (-want +got):", diff)
 	}
 }
 
@@ -417,7 +424,7 @@ func TestCreateRouteWithOneTargetReserve(t *testing.T) {
 	rev := Revision(testNamespace, "test-rev", MarkRevisionReady,
 		MarkInactive("NoTraffic", "no message"))
 
-	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(rev)
+	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(ctx, rev, metav1.CreateOptions{})
 	fakerevisioninformer.Get(ctx).Informer().GetIndexer().Add(rev)
 
 	// A configuration and associated revision. Normally the revision would be
@@ -426,10 +433,10 @@ func TestCreateRouteWithOneTargetReserve(t *testing.T) {
 	cfgrev := revisionForConfig(config)
 	config.Status.SetLatestCreatedRevisionName(cfgrev.Name)
 	config.Status.SetLatestReadyRevisionName(cfgrev.Name)
-	fakeservingclient.Get(ctx).ServingV1().Configurations(testNamespace).Create(config)
+	fakeservingclient.Get(ctx).ServingV1().Configurations(testNamespace).Create(ctx, config, metav1.CreateOptions{})
 	// Since Reconcile looks in the lister, we need to add it to the informer
 	fakecfginformer.Get(ctx).Informer().GetIndexer().Add(config)
-	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(cfgrev)
+	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(ctx, cfgrev, metav1.CreateOptions{})
 	fakerevisioninformer.Get(ctx).Informer().GetIndexer().Add(cfgrev)
 
 	// A route targeting both the config and standalone revision
@@ -442,7 +449,7 @@ func TestCreateRouteWithOneTargetReserve(t *testing.T) {
 			ConfigurationName: "test-config",
 			Percent:           ptr.Int64(10),
 		}))
-	fakeservingclient.Get(ctx).ServingV1().Routes(testNamespace).Create(route)
+	fakeservingclient.Get(ctx).ServingV1().Routes(testNamespace).Create(ctx, route, metav1.CreateOptions{})
 	// Since Reconcile looks in the lister, we need to add it to the informer
 	fakerouteinformer.Get(ctx).Informer().GetIndexer().Add(route)
 
@@ -454,7 +461,9 @@ func TestCreateRouteWithOneTargetReserve(t *testing.T) {
 		TLS: []v1alpha1.IngressTLS{},
 		Rules: []v1alpha1.IngressRule{{
 			Hosts: []string{
-				"test-route.test.svc.cluster.local",
+				"test-route.test",
+				"test-route.test.svc",
+				pkgnet.GetServiceHostname("test-route", "test"),
 			},
 			HTTP: &v1alpha1.HTTPIngressRuleValue{
 				Paths: []v1alpha1.HTTPIngressPath{{
@@ -481,7 +490,6 @@ func TestCreateRouteWithOneTargetReserve(t *testing.T) {
 							"Knative-Serving-Namespace": testNamespace,
 						},
 					}},
-					Timeout: &metav1.Duration{Duration: 48 * time.Hour},
 				}},
 			},
 			Visibility: v1alpha1.IngressVisibilityClusterLocal,
@@ -514,14 +522,13 @@ func TestCreateRouteWithOneTargetReserve(t *testing.T) {
 							"Knative-Serving-Namespace": testNamespace,
 						},
 					}},
-					Timeout: &metav1.Duration{Duration: 48 * time.Hour},
 				}},
 			},
 			Visibility: v1alpha1.IngressVisibilityExternalIP,
 		}},
 	}
 	if diff := cmp.Diff(expectedSpec, ci.Spec); diff != "" {
-		t.Errorf("Unexpected rule spec diff (-want +got): %v", diff)
+		t.Error("Unexpected rule spec diff (-want +got):", diff)
 	}
 }
 
@@ -531,7 +538,7 @@ func TestCreateRouteWithDuplicateTargets(t *testing.T) {
 
 	// A standalone revision
 	rev := Revision(testNamespace, "test-rev", MarkRevisionReady, WithK8sServiceName("test-rev"))
-	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(rev)
+	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(ctx, rev, metav1.CreateOptions{})
 	fakerevisioninformer.Get(ctx).Informer().GetIndexer().Add(rev)
 
 	// A configuration and associated revision. Normally the revision would be
@@ -540,10 +547,10 @@ func TestCreateRouteWithDuplicateTargets(t *testing.T) {
 	cfgrev := revisionForConfig(config)
 	config.Status.SetLatestCreatedRevisionName(cfgrev.Name)
 	config.Status.SetLatestReadyRevisionName(cfgrev.Name)
-	fakeservingclient.Get(ctx).ServingV1().Configurations(testNamespace).Create(config)
+	fakeservingclient.Get(ctx).ServingV1().Configurations(testNamespace).Create(ctx, config, metav1.CreateOptions{})
 	// Since Reconcile looks in the lister, we need to add it to the informer
 	fakecfginformer.Get(ctx).Informer().GetIndexer().Add(config)
-	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(cfgrev)
+	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(ctx, cfgrev, metav1.CreateOptions{})
 	fakerevisioninformer.Get(ctx).Informer().GetIndexer().Add(cfgrev)
 
 	// A route with duplicate targets. These will be deduped.
@@ -573,7 +580,7 @@ func TestCreateRouteWithDuplicateTargets(t *testing.T) {
 			RevisionName: "test-rev",
 			Percent:      ptr.Int64(15),
 		}))
-	fakeservingclient.Get(ctx).ServingV1().Routes(testNamespace).Create(route)
+	fakeservingclient.Get(ctx).ServingV1().Routes(testNamespace).Create(ctx, route, metav1.CreateOptions{})
 	// Since Reconcile looks in the lister, we need to add it to the informer
 	fakerouteinformer.Get(ctx).Informer().GetIndexer().Add(route)
 
@@ -585,7 +592,9 @@ func TestCreateRouteWithDuplicateTargets(t *testing.T) {
 		TLS: []v1alpha1.IngressTLS{},
 		Rules: []v1alpha1.IngressRule{{
 			Hosts: []string{
-				"test-route.test.svc.cluster.local",
+				"test-route.test",
+				"test-route.test.svc",
+				pkgnet.GetServiceHostname("test-route", "test"),
 			},
 			HTTP: &v1alpha1.HTTPIngressRuleValue{
 				Paths: []v1alpha1.HTTPIngressPath{{
@@ -612,7 +621,6 @@ func TestCreateRouteWithDuplicateTargets(t *testing.T) {
 							"Knative-Serving-Namespace": testNamespace,
 						},
 					}},
-					Timeout: &metav1.Duration{Duration: 48 * time.Hour},
 				}},
 			},
 			Visibility: v1alpha1.IngressVisibilityClusterLocal,
@@ -645,13 +653,14 @@ func TestCreateRouteWithDuplicateTargets(t *testing.T) {
 							"Knative-Serving-Namespace": testNamespace,
 						},
 					}},
-					Timeout: &metav1.Duration{Duration: 48 * time.Hour},
 				}},
 			},
 			Visibility: v1alpha1.IngressVisibilityExternalIP,
 		}, {
 			Hosts: []string{
-				"test-revision-1-test-route.test.svc.cluster.local",
+				"test-revision-1-test-route.test",
+				"test-revision-1-test-route.test.svc",
+				pkgnet.GetServiceHostname("test-revision-1-test-route", "test"),
 			},
 			HTTP: &v1alpha1.HTTPIngressRuleValue{
 				Paths: []v1alpha1.HTTPIngressPath{{
@@ -667,7 +676,6 @@ func TestCreateRouteWithDuplicateTargets(t *testing.T) {
 							"Knative-Serving-Namespace": testNamespace,
 						},
 					}},
-					Timeout: &metav1.Duration{Duration: 48 * time.Hour},
 				}},
 			},
 			Visibility: v1alpha1.IngressVisibilityClusterLocal,
@@ -689,13 +697,14 @@ func TestCreateRouteWithDuplicateTargets(t *testing.T) {
 							"Knative-Serving-Namespace": testNamespace,
 						},
 					}},
-					Timeout: &metav1.Duration{Duration: 48 * time.Hour},
 				}},
 			},
 			Visibility: v1alpha1.IngressVisibilityExternalIP,
 		}, {
 			Hosts: []string{
-				"test-revision-2-test-route.test.svc.cluster.local",
+				"test-revision-2-test-route.test",
+				"test-revision-2-test-route.test.svc",
+				pkgnet.GetServiceHostname("test-revision-2-test-route", "test"),
 			},
 			HTTP: &v1alpha1.HTTPIngressRuleValue{
 				Paths: []v1alpha1.HTTPIngressPath{{
@@ -711,7 +720,6 @@ func TestCreateRouteWithDuplicateTargets(t *testing.T) {
 							"Knative-Serving-Namespace": testNamespace,
 						},
 					}},
-					Timeout: &metav1.Duration{Duration: 48 * time.Hour},
 				}},
 			},
 			Visibility: v1alpha1.IngressVisibilityClusterLocal,
@@ -733,7 +741,6 @@ func TestCreateRouteWithDuplicateTargets(t *testing.T) {
 							"Knative-Serving-Namespace": testNamespace,
 						},
 					}},
-					Timeout: &metav1.Duration{Duration: 48 * time.Hour},
 				}},
 			},
 			Visibility: v1alpha1.IngressVisibilityExternalIP,
@@ -742,7 +749,7 @@ func TestCreateRouteWithDuplicateTargets(t *testing.T) {
 
 	if diff := cmp.Diff(expectedSpec, ci.Spec); diff != "" {
 		fmt.Printf("%+v\n", ci.Spec)
-		t.Errorf("Unexpected rule spec diff (-want +got): %v", diff)
+		t.Error("Unexpected rule spec diff (-want +got):", diff)
 	}
 }
 
@@ -751,7 +758,7 @@ func TestCreateRouteWithNamedTargets(t *testing.T) {
 	defer cf()
 	// A standalone revision
 	rev := Revision(testNamespace, "test-rev", MarkRevisionReady, WithK8sServiceName("test-rev"))
-	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(rev)
+	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(ctx, rev, metav1.CreateOptions{})
 	fakerevisioninformer.Get(ctx).Informer().GetIndexer().Add(rev)
 
 	// A configuration and associated revision. Normally the revision would be
@@ -760,10 +767,10 @@ func TestCreateRouteWithNamedTargets(t *testing.T) {
 	cfgrev := revisionForConfig(config)
 	config.Status.SetLatestCreatedRevisionName(cfgrev.Name)
 	config.Status.SetLatestReadyRevisionName(cfgrev.Name)
-	fakeservingclient.Get(ctx).ServingV1().Configurations(testNamespace).Create(config)
+	fakeservingclient.Get(ctx).ServingV1().Configurations(testNamespace).Create(ctx, config, metav1.CreateOptions{})
 	// Since Reconcile looks in the lister, we need to add it to the informer
 	fakecfginformer.Get(ctx).Informer().GetIndexer().Add(config)
-	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(cfgrev)
+	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(ctx, cfgrev, metav1.CreateOptions{})
 	fakerevisioninformer.Get(ctx).Informer().GetIndexer().Add(cfgrev)
 
 	// A route targeting both the config and standalone revision with named
@@ -779,7 +786,7 @@ func TestCreateRouteWithNamedTargets(t *testing.T) {
 			Percent:           ptr.Int64(50),
 		}))
 
-	fakeservingclient.Get(ctx).ServingV1().Routes(testNamespace).Create(route)
+	fakeservingclient.Get(ctx).ServingV1().Routes(testNamespace).Create(ctx, route, metav1.CreateOptions{})
 	// Since Reconcile looks in the lister, we need to add it to the informer
 	fakerouteinformer.Get(ctx).Informer().GetIndexer().Add(route)
 
@@ -791,7 +798,9 @@ func TestCreateRouteWithNamedTargets(t *testing.T) {
 		TLS: []v1alpha1.IngressTLS{},
 		Rules: []v1alpha1.IngressRule{{
 			Hosts: []string{
-				"test-route.test.svc.cluster.local",
+				"test-route.test",
+				"test-route.test.svc",
+				pkgnet.GetServiceHostname("test-route", "test"),
 			},
 			HTTP: &v1alpha1.HTTPIngressRuleValue{
 				Paths: []v1alpha1.HTTPIngressPath{{
@@ -818,7 +827,6 @@ func TestCreateRouteWithNamedTargets(t *testing.T) {
 							"Knative-Serving-Namespace": testNamespace,
 						},
 					}},
-					Timeout: &metav1.Duration{Duration: 48 * time.Hour},
 				}},
 			},
 			Visibility: v1alpha1.IngressVisibilityClusterLocal,
@@ -851,13 +859,14 @@ func TestCreateRouteWithNamedTargets(t *testing.T) {
 							"Knative-Serving-Namespace": testNamespace,
 						},
 					}},
-					Timeout: &metav1.Duration{Duration: 48 * time.Hour},
 				}},
 			},
 			Visibility: v1alpha1.IngressVisibilityExternalIP,
 		}, {
 			Hosts: []string{
-				"bar-test-route.test.svc.cluster.local",
+				"bar-test-route.test",
+				"bar-test-route.test.svc",
+				pkgnet.GetServiceHostname("bar-test-route", "test"),
 			},
 			HTTP: &v1alpha1.HTTPIngressRuleValue{
 				Paths: []v1alpha1.HTTPIngressPath{{
@@ -873,7 +882,6 @@ func TestCreateRouteWithNamedTargets(t *testing.T) {
 							"Knative-Serving-Namespace": testNamespace,
 						},
 					}},
-					Timeout: &metav1.Duration{Duration: 48 * time.Hour},
 				}},
 			},
 			Visibility: v1alpha1.IngressVisibilityClusterLocal,
@@ -895,13 +903,14 @@ func TestCreateRouteWithNamedTargets(t *testing.T) {
 							"Knative-Serving-Namespace": testNamespace,
 						},
 					}},
-					Timeout: &metav1.Duration{Duration: 48 * time.Hour},
 				}},
 			},
 			Visibility: v1alpha1.IngressVisibilityExternalIP,
 		}, {
 			Hosts: []string{
-				"foo-test-route.test.svc.cluster.local",
+				"foo-test-route.test",
+				"foo-test-route.test.svc",
+				pkgnet.GetServiceHostname("foo-test-route", "test"),
 			},
 			HTTP: &v1alpha1.HTTPIngressRuleValue{
 				Paths: []v1alpha1.HTTPIngressPath{{
@@ -917,7 +926,6 @@ func TestCreateRouteWithNamedTargets(t *testing.T) {
 							"Knative-Serving-Namespace": testNamespace,
 						},
 					}},
-					Timeout: &metav1.Duration{Duration: 48 * time.Hour},
 				}},
 			},
 			Visibility: v1alpha1.IngressVisibilityClusterLocal,
@@ -939,7 +947,6 @@ func TestCreateRouteWithNamedTargets(t *testing.T) {
 							"Knative-Serving-Namespace": testNamespace,
 						},
 					}},
-					Timeout: &metav1.Duration{Duration: 48 * time.Hour},
 				}},
 			},
 			Visibility: v1alpha1.IngressVisibilityExternalIP,
@@ -957,16 +964,16 @@ func TestCreateRouteWithNamedTargetsAndTagBasedRouting(t *testing.T) {
 
 	watcher.OnChange(&corev1.ConfigMap{
 		ObjectMeta: metav1.ObjectMeta{
-			Name:      network.ConfigName,
+			Name:      cfgmap.FeaturesConfigName,
 			Namespace: system.Namespace(),
 		},
 		Data: map[string]string{
-			network.TagHeaderBasedRoutingKey: "enabled",
+			"tag-header-based-routing": "enabled",
 		},
 	})
 	// A standalone revision
 	rev := Revision(testNamespace, "test-rev", MarkRevisionReady, WithK8sServiceName("test-rev"))
-	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(rev)
+	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(ctx, rev, metav1.CreateOptions{})
 	fakerevisioninformer.Get(ctx).Informer().GetIndexer().Add(rev)
 
 	// A configuration and associated revision. Normally the revision would be
@@ -975,10 +982,10 @@ func TestCreateRouteWithNamedTargetsAndTagBasedRouting(t *testing.T) {
 	cfgrev := revisionForConfig(config)
 	config.Status.SetLatestCreatedRevisionName(cfgrev.Name)
 	config.Status.SetLatestReadyRevisionName(cfgrev.Name)
-	fakeservingclient.Get(ctx).ServingV1().Configurations(testNamespace).Create(config)
+	fakeservingclient.Get(ctx).ServingV1().Configurations(testNamespace).Create(ctx, config, metav1.CreateOptions{})
 	// Since Reconcile looks in the lister, we need to add it to the informer
 	fakecfginformer.Get(ctx).Informer().GetIndexer().Add(config)
-	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(cfgrev)
+	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(ctx, cfgrev, metav1.CreateOptions{})
 	fakerevisioninformer.Get(ctx).Informer().GetIndexer().Add(cfgrev)
 
 	// A route targeting both the config and standalone revision with named
@@ -994,10 +1001,9 @@ func TestCreateRouteWithNamedTargetsAndTagBasedRouting(t *testing.T) {
 			Percent:           ptr.Int64(50),
 		}))
 
-	fakeservingclient.Get(ctx).ServingV1().Routes(testNamespace).Create(route)
+	fakeservingclient.Get(ctx).ServingV1().Routes(testNamespace).Create(ctx, route, metav1.CreateOptions{})
 	// Since Reconcile looks in the lister, we need to add it to the informer
 	fakerouteinformer.Get(ctx).Informer().GetIndexer().Add(route)
-
 	ctl.Reconciler.Reconcile(context.Background(), KeyOrDie(route))
 
 	ci := getRouteIngressFromClient(ctx, t, route)
@@ -1006,7 +1012,9 @@ func TestCreateRouteWithNamedTargetsAndTagBasedRouting(t *testing.T) {
 		TLS: []v1alpha1.IngressTLS{},
 		Rules: []v1alpha1.IngressRule{{
 			Hosts: []string{
-				"test-route.test.svc.cluster.local",
+				"test-route.test",
+				"test-route.test.svc",
+				pkgnet.GetServiceHostname("test-route", "test"),
 			},
 			HTTP: &v1alpha1.HTTPIngressRuleValue{
 				Paths: []v1alpha1.HTTPIngressPath{{
@@ -1029,7 +1037,6 @@ func TestCreateRouteWithNamedTargetsAndTagBasedRouting(t *testing.T) {
 							},
 						},
 					},
-					Timeout: &metav1.Duration{Duration: 48 * time.Hour},
 				}, {
 					Headers: map[string]v1alpha1.HeaderMatch{
 						network.TagHeaderName: {
@@ -1050,7 +1057,6 @@ func TestCreateRouteWithNamedTargetsAndTagBasedRouting(t *testing.T) {
 							},
 						},
 					},
-					Timeout: &metav1.Duration{Duration: 48 * time.Hour},
 				}, {
 					AppendHeaders: map[string]string{
 						network.DefaultRouteHeaderName: "true",
@@ -1078,7 +1084,6 @@ func TestCreateRouteWithNamedTargetsAndTagBasedRouting(t *testing.T) {
 							"Knative-Serving-Namespace": testNamespace,
 						},
 					}},
-					Timeout: &metav1.Duration{Duration: 48 * time.Hour},
 				}},
 			},
 			Visibility: v1alpha1.IngressVisibilityClusterLocal,
@@ -1107,7 +1112,6 @@ func TestCreateRouteWithNamedTargetsAndTagBasedRouting(t *testing.T) {
 							},
 						},
 					},
-					Timeout: &metav1.Duration{Duration: 48 * time.Hour},
 				}, {
 					Headers: map[string]v1alpha1.HeaderMatch{
 						network.TagHeaderName: {
@@ -1128,7 +1132,6 @@ func TestCreateRouteWithNamedTargetsAndTagBasedRouting(t *testing.T) {
 							},
 						},
 					},
-					Timeout: &metav1.Duration{Duration: 48 * time.Hour},
 				}, {
 					AppendHeaders: map[string]string{
 						network.DefaultRouteHeaderName: "true",
@@ -1156,13 +1159,14 @@ func TestCreateRouteWithNamedTargetsAndTagBasedRouting(t *testing.T) {
 							"Knative-Serving-Namespace": testNamespace,
 						},
 					}},
-					Timeout: &metav1.Duration{Duration: 48 * time.Hour},
 				}},
 			},
 			Visibility: v1alpha1.IngressVisibilityExternalIP,
 		}, {
 			Hosts: []string{
-				"bar-test-route.test.svc.cluster.local",
+				"bar-test-route.test",
+				"bar-test-route.test.svc",
+				pkgnet.GetServiceHostname("bar-test-route", "test"),
 			},
 			HTTP: &v1alpha1.HTTPIngressRuleValue{
 				Paths: []v1alpha1.HTTPIngressPath{{
@@ -1181,7 +1185,6 @@ func TestCreateRouteWithNamedTargetsAndTagBasedRouting(t *testing.T) {
 					AppendHeaders: map[string]string{
 						network.TagHeaderName: "bar",
 					},
-					Timeout: &metav1.Duration{Duration: 48 * time.Hour},
 				}},
 			},
 			Visibility: v1alpha1.IngressVisibilityClusterLocal,
@@ -1206,13 +1209,14 @@ func TestCreateRouteWithNamedTargetsAndTagBasedRouting(t *testing.T) {
 					AppendHeaders: map[string]string{
 						network.TagHeaderName: "bar",
 					},
-					Timeout: &metav1.Duration{Duration: 48 * time.Hour},
 				}},
 			},
 			Visibility: v1alpha1.IngressVisibilityExternalIP,
 		}, {
 			Hosts: []string{
-				"foo-test-route.test.svc.cluster.local",
+				"foo-test-route.test",
+				"foo-test-route.test.svc",
+				pkgnet.GetServiceHostname("foo-test-route", "test"),
 			},
 			HTTP: &v1alpha1.HTTPIngressRuleValue{
 				Paths: []v1alpha1.HTTPIngressPath{{
@@ -1228,7 +1232,6 @@ func TestCreateRouteWithNamedTargetsAndTagBasedRouting(t *testing.T) {
 							"Knative-Serving-Namespace": testNamespace,
 						},
 					}},
-					Timeout: &metav1.Duration{Duration: 48 * time.Hour},
 					AppendHeaders: map[string]string{
 						network.TagHeaderName: "foo",
 					},
@@ -1253,7 +1256,6 @@ func TestCreateRouteWithNamedTargetsAndTagBasedRouting(t *testing.T) {
 							"Knative-Serving-Namespace": testNamespace,
 						},
 					}},
-					Timeout: &metav1.Duration{Duration: 48 * time.Hour},
 					AppendHeaders: map[string]string{
 						network.TagHeaderName: "foo",
 					},
@@ -1265,7 +1267,7 @@ func TestCreateRouteWithNamedTargetsAndTagBasedRouting(t *testing.T) {
 
 	if diff := cmp.Diff(expectedSpec, ci.Spec); diff != "" {
 		fmt.Printf("%+v\n", ci.Spec)
-		t.Errorf("Unexpected rule spec diff (-want +got): %v", diff)
+		t.Error("Unexpected rule spec diff (-want +got):", diff)
 	}
 }
 
@@ -1301,7 +1303,7 @@ func TestUpdateDomainConfigMap(t *testing.T) {
 				"mytestdomain.com": "selector:\n  app: prod",
 			}
 			watcher.OnChange(&templateCM)
-			r.Labels = make(map[string]string)
+			r.Labels = nil
 		},
 	}, {
 		// When no domain with an open selector is specified, we fallback
@@ -1312,7 +1314,7 @@ func TestUpdateDomainConfigMap(t *testing.T) {
 				"mytestdomain.com": "selector:\n  app: prod",
 			}
 			watcher.OnChange(&templateCM)
-			r.Labels = make(map[string]string)
+			r.Labels = nil
 		},
 	}}
 
@@ -1327,27 +1329,26 @@ func TestUpdateDomainConfigMap(t *testing.T) {
 				cf()
 				waitInformers()
 			}()
-			route := Route(testNamespace, "test-route", WithRouteLabel(map[string]string{"route": "test-route"}))
-			route.Name = uuid.New().String()
-			route.Labels = map[string]string{"app": "prod"}
+			route := Route(testNamespace, "test", WithRouteGeneration(1982),
+				WithRouteLabel(map[string]string{"app": "prod"}))
 			routeClient := fakeservingclient.Get(ctx).ServingV1().Routes(route.Namespace)
 
 			// Create a route.
 			fakerouteinformer.Get(ctx).Informer().GetIndexer().Add(route)
-			routeClient.Create(route)
+			routeClient.Create(ctx, route, metav1.CreateOptions{})
 			if err := ctl.Reconciler.Reconcile(context.Background(), KeyOrDie(route)); err != nil {
 				t.Fatal("Reconcile() =", err)
 			}
 			addRouteToInformers(ctx, t, route)
 
 			// Wait initial reconcile to finish.
-			rl := fakerouteinformer.Get(ctx).Lister()
+			rl := fakerouteinformer.Get(ctx).Lister().Routes(route.Namespace)
 			if err := wait.PollImmediate(10*time.Millisecond, 5*time.Second, func() (bool, error) {
-				r, err := rl.Routes(route.Namespace).Get(route.Name)
+				r, err := rl.Get(route.Name)
 				if err != nil {
 					return false, err
 				}
-				return !cmp.Equal(r.Status, route.Status), nil
+				return !cmp.Equal(r.Status, route.Status) && r.Generation == r.Status.ObservedGeneration, nil
 			}); err != nil {
 				t.Fatal("Failed to see route initial reconcile propagation:", err)
 			}
@@ -1356,33 +1357,57 @@ func TestUpdateDomainConfigMap(t *testing.T) {
 			// generates an actual diff. Otherwise the value in the
 			// fake cache is going to be changed. Now, this update sometimes races with
 			// the CM update and since the update below is a noop (the same object
-			// is applied), noting happens and the test fails.
-			route = route.DeepCopy()
-			tc.apply(route, watcher)
-			fakerouteinformer.Get(ctx).Informer().GetIndexer().Add(route)
-			if _, err := routeClient.Update(route); err != nil {
-				t.Fatal("Route.Update() =", err)
-			}
-
-			if err := ctl.Reconciler.Reconcile(context.Background(), KeyOrDie(route)); err != nil {
-				t.Fatal("Reconcile() =", err)
-			}
-
+			// is applied), nothing happens and the test fails (for one of the tests).
+			// So we'll retry the operation several times.
+			timeout := time.After(10 * time.Second)
 			wantDomain := fmt.Sprintf("%s.%s.%s", route.Name, route.Namespace, tc.expectedDomainSuffix)
-			var gotDomain string
-			if err := wait.PollImmediate(10*time.Millisecond, 5*time.Second, func() (bool, error) {
-				r, err := routeClient.Get(route.Name, metav1.GetOptions{})
-				if err != nil {
-					return false, err
+			for i := 1; ; i++ {
+				select {
+				case <-timeout:
+					t.Fatal("Timed out trying to see the propagation")
+				default:
+					t.Log("Starting iteration", i)
 				}
-				// Wait for the domain to propagate.
-				if r.Status.URL == nil {
-					return false, nil
+				route, _ := rl.Get(route.Name)
+				route = route.DeepCopy()
+				route.Generation++
+				tc.apply(route, watcher)
+				if _, err := routeClient.Update(ctx, route, metav1.UpdateOptions{}); err != nil {
+					t.Fatal("Route.Update() =", err)
+				}
+
+				// Ensure we have the proper version in the informers.
+				if err := wait.PollImmediate(10*time.Millisecond, 3*time.Second, func() (bool, error) {
+					r, err := rl.Get(route.Name)
+					return r != nil && r.Generation == route.Generation, err
+				}); err != nil {
+					t.Fatal("Failed to see informers get the new Route version:", err)
+				}
+
+				// Now that we know the exact version the reconciler is going to see in the
+				// informers, let's reconcile.
+				if err := ctl.Reconciler.Reconcile(context.Background(), KeyOrDie(route)); err != nil {
+					t.Fatal("Reconcile() =", err)
+				}
+
+				var gotDomain string
+				if err := wait.PollImmediate(10*time.Millisecond, 5*time.Second, func() (bool, error) {
+					r, err := routeClient.Get(ctx, route.Name, metav1.GetOptions{})
+					if err != nil {
+						return false, err
+					}
+					// Wait for the object to reconcile and the domain to propagate.
+					if r.Status.ObservedGeneration != route.Generation && r.Status.URL == nil {
+						return false, nil
+					}
+					gotDomain = r.Status.URL.Host
+					return gotDomain == wantDomain, nil
+				}); err != nil {
+					t.Logf("Failed to see route domain propagation, got %s, want %s: %v", gotDomain, wantDomain, err)
+				} else {
+					t.Log("Success")
+					break
 				}
-				gotDomain = r.Status.URL.Host
-				return gotDomain == wantDomain, nil
-			}); err != nil {
-				t.Fatalf("Failed to see route domain propagation, got %s, want %s: %v", gotDomain, wantDomain, err)
 			}
 		})
 	}
@@ -1460,7 +1485,7 @@ func TestGlobalResyncOnUpdateDomainConfigMap(t *testing.T) {
 			defer func() {
 				cf()
 				if err := grp.Wait(); err != nil {
-					t.Errorf("Wait() = %v", err)
+					t.Error("Wait() =", err)
 				}
 				waitInformers()
 			}()
@@ -1475,7 +1500,7 @@ func TestGlobalResyncOnUpdateDomainConfigMap(t *testing.T) {
 			route := Route(testNamespace, "test-route",
 				WithRouteLabel(map[string]string{"app": "prod"}), WithRouteGeneration(1))
 
-			created, err := servingClient.ServingV1().Routes(route.Namespace).Create(route)
+			created, err := servingClient.ServingV1().Routes(route.Namespace).Create(ctx, route, metav1.CreateOptions{})
 			if err != nil {
 				t.Fatal("Failed to create route", err)
 			}
@@ -1513,7 +1538,7 @@ func TestGlobalResyncOnUpdateDomainConfigMap(t *testing.T) {
 func TestRouteDomain(t *testing.T) {
 	route := Route("default", "myapp", WithRouteLabel(map[string]string{"route": "myapp"}), WithRouteAnnotation(map[string]string{"sub": "mysub"}))
 	ctx := context.Background()
-	cfg := ReconcilerTestConfig(false)
+	cfg := reconcilerTestConfig(false)
 	ctx = config.ToContext(ctx, cfg)
 
 	tests := []struct {
@@ -1537,10 +1562,10 @@ func TestRouteDomain(t *testing.T) {
 		Pass:     true,
 		Expected: "myapp.example.com",
 	}, {
-		Name:     "SuperShort",
+		Name:     "Too short", // domain must be at least two segments separated by dots.
 		Template: "{{.Name}}",
-		Pass:     true,
-		Expected: "myapp",
+		Pass:     false,
+		Expected: "",
 	}, {
 		Name:     "Annotations",
 		Template: `{{.Name}}.{{ index .Annotations "sub"}}.{{.Domain}}`,
@@ -1569,3 +1594,59 @@ func TestRouteDomain(t *testing.T) {
 		})
 	}
 }
+
+func TestAutoTLSEnabled(t *testing.T) {
+	tests := []struct {
+		name                  string
+		configAutoTLSEnabled  bool
+		tlsDisabledAnnotation string
+		wantAutoTLSEnabled    bool
+	}{{
+		name:                 "AutoTLS enabled by config, not disabled by annotation",
+		configAutoTLSEnabled: true,
+		wantAutoTLSEnabled:   true,
+	}, {
+		name:                  "AutoTLS enabled by config, disabled by annotation",
+		configAutoTLSEnabled:  true,
+		tlsDisabledAnnotation: "true",
+		wantAutoTLSEnabled:    false,
+	}, {
+		name:                 "AutoTLS disabled by config, not disabled by annotation",
+		configAutoTLSEnabled: false,
+		wantAutoTLSEnabled:   false,
+	}, {
+		name:                  "AutoTLS disabled by config, disabled by annotation",
+		configAutoTLSEnabled:  false,
+		tlsDisabledAnnotation: "true",
+		wantAutoTLSEnabled:    false,
+	}, {
+		name:                  "AutoTLS enabled by config, invalid annotation",
+		configAutoTLSEnabled:  true,
+		tlsDisabledAnnotation: "foo",
+		wantAutoTLSEnabled:    true,
+	}, {
+		name:                  "AutoTLS disabled by config, invalid annotation",
+		configAutoTLSEnabled:  false,
+		tlsDisabledAnnotation: "foo",
+		wantAutoTLSEnabled:    false,
+	}}
+
+	r := Route("test-ns", "test-route")
+	r.Annotations = map[string]string{}
+
+	for _, test := range tests {
+		t.Run(test.name, func(t *testing.T) {
+			ctx := config.ToContext(context.Background(), &config.Config{
+				Network: &network.Config{
+					AutoTLS: test.configAutoTLSEnabled,
+				},
+			})
+
+			r.Annotations[networking.DisableAutoTLSAnnotationKey] = test.tlsDisabledAnnotation
+
+			if got := autoTLSEnabled(ctx, r); got != test.wantAutoTLSEnabled {
+				t.Errorf("autoTLSEnabled = %t, want %t", got, test.wantAutoTLSEnabled)
+			}
+		})
+	}
+}
