@@ -19,6 +19,7 @@ limitations under the License.
 package v1
 
 import (
+	"context"
 	"fmt"
 	"net/url"
 	"regexp"
@@ -83,7 +84,7 @@ func canServeRequests(t *testing.T, clients *test.Clients, route *v1.Route) erro
 		route.Name,
 		func(r *v1.Route) (bool, error) {
 			url = r.Status.URL.URL()
-			return url != nil, nil
+			return url.String() != "", nil
 		},
 		"RouteDomain",
 	)
@@ -93,13 +94,14 @@ func canServeRequests(t *testing.T, clients *test.Clients, route *v1.Route) erro
 
 	t.Logf("Route %s can serve the expected data at %s", route.Name, url)
 	_, err = pkgTest.WaitForEndpointState(
+		context.Background(),
 		clients.KubeClient,
 		t.Logf,
 		url,
 		v1test.RetryingRouteInconsistency(pkgTest.MatchesAllOf(pkgTest.IsStatusOK, pkgTest.MatchesBody(test.HelloWorldText))),
 		"WaitForEndpointToServeText",
 		test.ServingFlags.ResolvableDomain,
-		test.AddRootCAtoTransport(t.Logf, clients, test.ServingFlags.Https))
+		test.AddRootCAtoTransport(context.Background(), t.Logf, clients, test.ServingFlags.HTTPS))
 	if err != nil {
 		return fmt.Errorf("the endpoint for Route %s at %s didn't serve the expected text %q: %w", route.Name, url, test.HelloWorldText, err)
 	}
@@ -120,11 +122,10 @@ func TestServiceGenerateName(t *testing.T) {
 	}
 
 	// Cleanup on test failure.
-	test.CleanupOnInterrupt(func() { test.TearDown(clients, names) })
-	defer func() { test.TearDown(clients, names) }()
+	test.EnsureTearDown(t, clients, &names)
 
 	// Create the service using the generate name field. If the service does not become ready this will fail.
-	t.Logf("Creating new service with generateName %s", generateName)
+	t.Log("Creating new service with generateName", generateName)
 	resources, err := v1test.CreateServiceReady(t, clients, &names, setServiceGenerateName(generateName))
 	if err != nil {
 		t.Fatalf("Failed to create service with generateName %s: %v", generateName, err)
@@ -156,10 +157,9 @@ func TestRouteAndConfigGenerateName(t *testing.T) {
 		Image: test.HelloWorld,
 	}
 
-	test.CleanupOnInterrupt(func() { test.TearDown(clients, names) })
-	defer func() { test.TearDown(clients, names) }()
+	test.EnsureTearDown(t, clients, &names)
 
-	t.Logf("Creating new configuration with generateName %s", generateName)
+	t.Log("Creating new configuration with generateName", generateName)
 	config, err := v1test.CreateConfiguration(t, clients, names, setConfigurationGenerateName(generateName))
 	if err != nil {
 		t.Fatalf("Failed to create configuration with generateName %s: %v", generateName, err)
@@ -180,7 +180,7 @@ func TestRouteAndConfigGenerateName(t *testing.T) {
 	}
 
 	// Create a route that maps to the revision created by the configuration above
-	t.Logf("Create new Route with generateName %s", generateName)
+	t.Log("Create new Route with generateName", generateName)
 	route, err := v1test.CreateRoute(t, clients, names, setRouteGenerateName(generateName))
 	if err != nil {
 		t.Fatalf("Failed to create route with generateName %s: %v", generateName, err)
